"""
Generate test code tool

copyrigth https://github.com/shigeshige/py-ut-generator
定数一覧
"""

AST_EXPR = 'Expr'
AST_FUCNTION = 'FunctionDef'
AST_FOR = 'For'
AST_IF = 'If'
AST_IMPORT_FROM = 'ImportFrom'
AST_IMPORT = 'Import'
AST_MODULE = 'Module'
AST_RETURN = 'Return'
AST_CALL = 'Call'
AST_NAME = 'Name'
AST_ATTRIBUTE = 'Attribute'
AST_ASSIGN = 'Assign'
AST_WHILE = 'While'
AST_WITH = 'With'
AST_CLASS = 'ClassDef'

STR_PRE_FUNC = 'test_'
